@@ -1,11 +1,11 @@
 use strict;
 use warnings;
 package Tapper::Reports::Receiver::Level2::BenchmarkAnything;
-# git description: v5.0.0-3-g727e7df
+# git description: v5.0.1-7-gef05d72
 
 our $AUTHORITY = 'cpan:SCHWIGON';
 # ABSTRACT: Tapper - Level2 receiver plugin to forward BenchmarkAnything data
-$Tapper::Reports::Receiver::Level2::BenchmarkAnything::VERSION = '5.0.1';
+$Tapper::Reports::Receiver::Level2::BenchmarkAnything::VERSION = '5.0.2';
 use Try::Tiny;
 use Data::Dumper;
 use Data::DPath 'dpath';
@@ -16,17 +16,13 @@ use Tapper::Config 5.0.2; # 5.0.2 provides {_last_used_tapper_config_file}
 
 # TAP aggregation metrics per type
 our @boolean_aggregation_metrics  = qw(all_passed has_errors has_problems);
-our @counter_aggregation_metrics  = qw(failed parse_errors total passed skipped todo todo_passed wait exit);
+our @counter_aggregation_metrics  = qw(failed parse_errors total passed skipped todo todo_passed);
 our @textual_aggregation_metrics  = qw(status);
 
 
 sub submit {
   my ($util, $report, $options) = @_;
 
-  local $Data::Dumper::Pair = ":";
-  local $Data::Dumper::Terse = 2;
-  local $Data::Dumper::Sortkeys = 1;
-
   my $benchmark_entries_path          = $options->{benchmark_entries_path};
   my $additional_metainfo_path        = $options->{additional_metainfo_path};
   my $store_metainfo_as_benchmarks    = $options->{store_metainfo_as_benchmarks};
@@ -46,17 +42,17 @@ sub submit {
           {
             $section = $section->{section}{$section_name};
 
-            my $metric_name = $section_name;
-            $metric_name =~ s/\.t$//;
-            $metric_name =~ s/\W+/_/g;
-            $util->log->debug("section metric_name: $metric_name");
+            my $section_metric_name = $section_name;
+            $section_metric_name =~ s/\.t$//;
+            $section_metric_name =~ s/\W+/_/g;
+            $util->log->debug("section_metric_name: $section_metric_name");
 
             # Metric name contemplation:
             # --------------------------
             #
             # There is a decision to make between 1) having
-            # $metric_name as part of the NAME field or 2) having
-            # generic NAMEs with $metric_name as an attribute of
+            # $section_metric_name as part of the NAME field or 2) having
+            # generic NAMEs with $section_metric_name as an attribute of
             # it.
             #
             # Here we make it part of NAME (option 1).
@@ -89,21 +85,28 @@ sub submit {
             #
             foreach my $entry (@boolean_aggregation_metrics, @counter_aggregation_metrics, @textual_aggregation_metrics)
               {
-                push @test_metrics, { NAME  => "tap.summary.${metric_name}.${entry}",
+                push @test_metrics, { NAME  => "tap.summary.section.${section_metric_name}.${entry}",
                                       VALUE => $section->{tap}{summary}{$entry},
                                     };
               }
+            # success ratio
+            my $section_total  = $section->{tap}{summary}{total};
+            my $section_passed = $section->{tap}{summary}{passed};
+            my $section_ratio  = sprintf("%.2f", 100 * ($section_total == 0 ? 0 : $section_passed / $section_total));
+            push @test_metrics, { NAME  => "tap.summary.section.${section_metric_name}.success_ratio",
+                                  VALUE => $section_ratio,
+                                };
 
-            # summarize all sections
-            # - bool metrics -
+            # --- summarize all sections ---
+            # - bool metrics
             foreach my $entry (@boolean_aggregation_metrics) {
               $test_metrics_aggregated{$entry} ||= $section->{tap}{summary}{$entry};
             }
-            # - counter metrics -
+            # - counter metrics
             foreach my $entry (@counter_aggregation_metrics) {
               $test_metrics_aggregated{$entry} += ($section->{tap}{summary}{$entry} || 0);
             }
-            # - no textual aggregation -
+            # - no textual aggregations of "PASS"/"FAIL"/etc.
           }
       }
 
@@ -112,13 +115,36 @@ sub submit {
     # Note, that with "all.sections" containing a dot and the actual
     # metric names above not containing dots there are no collisions
     # in NAME.
+    my $suite_name = $report->suite->name || "unknown-suite-name";
     foreach my $entry (@boolean_aggregation_metrics, @counter_aggregation_metrics) {
-      push @test_metrics, { NAME  => "tap.summary.all.sections.${entry}",
+      push @test_metrics, { NAME  => "tap.summary.suite.${suite_name}.${entry}",
+                            VALUE => $test_metrics_aggregated{$entry},
+                          };
+      push @test_metrics, { NAME  => "tap.summary.all.${entry}", # same but regardless of actual suite
                             VALUE => $test_metrics_aggregated{$entry},
                           };
     }
 
-    $util->log->debug("test metrics: ".Dumper(\@test_metrics));
+    # success ratios
+    my $agg_total  = $test_metrics_aggregated{total};
+    my $agg_passed = $test_metrics_aggregated{passed};
+    my $agg_ratio  = sprintf("%.2f", 100 * ($agg_total == 0 ? 0 : $agg_passed / $agg_total));
+    push @test_metrics, { NAME  => "tap.summary.suite.${suite_name}.success_ratio",
+                          VALUE => $agg_ratio,
+                        };
+    push @test_metrics, { NAME  => "tap.summary.all.success_ratio",
+                          VALUE => $agg_ratio,
+                        };
+
+    {
+      # You MUST localize Data::Dumper settings to strictly ONLY cover
+      # the debug output - otherwise it wrongly serializes the tap_dom
+      # cache into the DB during get_cached_tapdom().
+      local $Data::Dumper::Pair = ":";
+      local $Data::Dumper::Terse = 2;
+      local $Data::Dumper::Sortkeys = 1;
+      $util->log->debug("test metrics: ".Dumper(\@test_metrics));
+    }
 
     my @benchmark_entries = dpath($benchmark_entries_path)->match($tap_dom);
     @benchmark_entries = @{$benchmark_entries[0]} while $benchmark_entries[0] && reftype $benchmark_entries[0] eq "ARRAY"; # deref all array envelops
@@ -156,8 +182,19 @@ sub submit {
         $entry->{tapper_testrun} ||= $report->reportgrouptestrun->testrun_id if $report->reportgrouptestrun && $report->reportgrouptestrun->testrun_id;
         $entry->{tapper_reportgroup_arbitrary} ||= $report->reportgrouparbitrary->arbitrary_id if $report->reportgrouparbitrary && $report->reportgrouparbitrary->arbitrary_id;
 
+        # mark benchmkars from failed reports (and only them, for space-saving reasons)
+        $entry->{tapper_report_success} = ( $agg_ratio < 100 ? 0 : 1) if !defined $entry->{tapper_report_success};
+
         # debug log
-        $util->log->debug("store benchmark: ".Dumper($entry));
+        {
+          # You MUST localize Data::Dumper settings to strictly ONLY cover
+          # the debug output - otherwise it wrongly serializes the tap_dom
+          # cache into the DB during get_cached_tapdom().
+          local $Data::Dumper::Pair = ":";
+          local $Data::Dumper::Terse = 2;
+          local $Data::Dumper::Sortkeys = 1;
+          $util->log->debug("store benchmark: ".Dumper($entry));
+        }
 
         # actually submit data
         $balib->add ({BenchmarkAnythingData => [$entry]});
@@ -186,7 +223,15 @@ sub submit {
             $entry->{tapper_reportgroup_arbitrary} ||= $report->reportgrouparbitrary->arbitrary_id if $report->reportgrouparbitrary && $report->reportgrouparbitrary->arbitrary_id;
 
             # debug log
-            $util->log->debug("store test metric: ".Dumper($entry));
+            {
+              # You MUST localize Data::Dumper settings to strictly ONLY cover
+              # the debug output - otherwise it wrongly serializes the tap_dom
+              # cache into the DB during get_cached_tapdom().
+              local $Data::Dumper::Pair = ":";
+              local $Data::Dumper::Terse = 2;
+              local $Data::Dumper::Sortkeys = 1;
+              $util->log->debug("store test metric: ".Dumper($entry));
+            }
 
             # actually submit data
             $balib->add ({BenchmarkAnythingData => [$entry]});
@@ -199,7 +244,15 @@ sub submit {
       {
         foreach my $entry (@metainfo_entries)
           {
-            $util->log->debug("store metainfo: ".Dumper($entry));
+            {
+              # You MUST localize Data::Dumper settings to strictly ONLY cover
+              # the debug output - otherwise it wrongly serializes the tap_dom
+              # cache into the DB during get_cached_tapdom().
+              local $Data::Dumper::Pair = ":";
+              local $Data::Dumper::Terse = 2;
+              local $Data::Dumper::Sortkeys = 1;
+              $util->log->debug("store metainfo: ".Dumper($entry));
+            }
             $balib->add ({BenchmarkAnythingData => [$entry]});
             $benchmark_counter++;
           }
